//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

float blur <string uiname="Blur Amount";> =0.0060;
float bal  <string uiname="Brightness Balance";> =0.0000;
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};


texture ctTex <string uiname="Control Texture";>;
sampler ctSamp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (ctTex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd : TEXCOORD0;
};


// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    vs2ps Out = (vs2ps)0;
    Out.Pos = mul(Pos, tWVP);
    Out.TexCd = mul(TexCd, tTex);
    return Out;
}
// PIXELSHADERS:
//------------------------------------------------------------
float4 showorig(vs2ps In): COLOR
{
   float4 orig = tex2D(Samp, In.TexCd);
   return orig;
}
//------------------------------------------------------------
float4 showct(vs2ps In): COLOR
{
   float4 ctrl = tex2D(ctSamp, In.TexCd);
   return ctrl;
}
//------------------------------------------------------------
float4 blackblur(vs2ps In): COLOR
{
   float4 col = tex2D(Samp, In.TexCd);
   float4 orig = col;
   float4 ctrl = tex2D(ctSamp, In.TexCd);
   float2 offset[13] =
   {    0,     0, -0.75,-0.75,   -1,     0, -0.75, 0.75,
        0,    -1,     0,    1, 0.75, -0.75,     1,    0,
     0.75,  0.75,  -1.6,    0,  1.6,     0,     0, -1.6,
        0,   1.6
   };
   float weight[13] = {4, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1};
    col.rgb = 0;

    for(int i=0; i<13; i++)
    {
    col += tex2D(Samp, In.TexCd + offset[i] * blur) * weight[i] / 21;
    }
    col= col *(1-ctrl)*(1-bal) + orig * ctrl*(1+bal);
   return col;
}
//------------------------------------------------------------
float4 whiteblur(vs2ps In): COLOR
{
   float4 col = tex2D(Samp, In.TexCd);
   float4 orig = col;
   float4 ctrl = tex2D(ctSamp, In.TexCd);
   float2 offset[13] =
   {    0,     0, -0.75,-0.75,   -1,     0, -0.75, 0.75,
        0,    -1,     0,    1, 0.75, -0.75,     1,    0,
     0.75,  0.75,  -1.6,    0,  1.6,     0,     0, -1.6,
        0,   1.6
   };
   float weight[13] = {4, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1};
    col.rgb = 0;

    for(int i=0; i<13; i++)
    {
    col += tex2D(Samp, In.TexCd + offset[i] * blur) * weight[i] / 21;
    }
    col= col * ctrl * (1+bal) + orig * (1-ctrl)*(1-bal);
   return col;
}


// TECHNIQUES:
technique ShowOriginalTextureOnly
{
    pass P0
    {
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        VertexShader = compile vs_1_0 VS();
        PixelShader  = compile ps_1_0 showorig();
    }


}
technique ShowControlTextureOnly
{
    pass P0
    {
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        VertexShader = compile vs_1_0 VS();
        PixelShader  = compile ps_1_0 showct();
    }


}

technique BlurBlack
{
    pass P0
    {
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        VertexShader = compile vs_1_0 VS();
        PixelShader  = compile ps_2_0 blackblur();
    }

    
}

technique BlurWhite
{
    pass P0
    {
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        VertexShader = compile vs_1_0 VS();
        PixelShader  = compile ps_2_0 whiteblur();
    }


}
